Get a data.frame with columns equal to [B, Q(B), std.err(Q(B)), i, k], where
i is the unit and k the treatment arm that is optimal to assign at a spend level B.
Usage
# S3 method for maq
summary(object, ...)
Value
A data.frame making up the elements of the estimated Qini curve.